"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OdooTrigger = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const GenericFunctionsTrigger_1 = require("./GenericFunctionsTrigger");
const version_1 = require("../version");
class OdooTrigger {
    constructor() {
        this.description = {
            displayName: 'Odoo Trigger',
            name: 'odooTrigger',
            icon: 'file:odoo.svg',
            group: ['trigger'],
            version: 1,
            description: `Handle Odoo events via webhooks (v.${version_1.version})`,
            defaults: {
                name: 'Odoo-Trigger',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [
                {
                    name: 'odooApi',
                    required: true,
                    testedBy: 'odooApiTest',
                },
            ],
            webhooks: [
                {
                    name: 'default',
                    httpMethod: 'POST',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
            ],
            properties: [
                {
                    displayName: 'Resource Name or ID',
                    name: 'odooResource',
                    type: 'options',
                    description: 'Choose from the list, or specify an ID using an <a href="https://docs.n8n.io/code-examples/expressions/">expression</a>',
                    default: '',
                    typeOptions: {
                        loadOptionsMethod: 'getModels',
                    },
                },
                {
                    displayName: 'Trigger',
                    name: 'trigger',
                    type: 'options',
                    default: 'on_create',
                    options: [
                        { name: 'On Create', value: 'on_create' },
                        { name: 'On Update', value: 'on_write' },
                        { name: 'On Create And Update', value: 'on_create_or_write' },
                        { name: 'On Delete', value: 'on_unlink' },
                    ],
                    description: 'The method the webhook should trigger',
                },
            ],
        };
        this.methods = {
            loadOptions: {
                async getModels() {
                    const credentials = await this.getCredentials('odooApi');
                    const url = credentials === null || credentials === void 0 ? void 0 : credentials.url;
                    const username = credentials === null || credentials === void 0 ? void 0 : credentials.username;
                    const password = credentials === null || credentials === void 0 ? void 0 : credentials.password;
                    const db = (0, GenericFunctionsTrigger_1.odooGetDBName)(credentials === null || credentials === void 0 ? void 0 : credentials.db, url);
                    const userID = await GenericFunctionsTrigger_1.odooGetUserID.call(this, db, username, password, url);
                    const body = {
                        jsonrpc: '2.0',
                        method: 'call',
                        params: {
                            service: 'object',
                            method: 'execute',
                            args: [
                                db,
                                userID,
                                password,
                                'ir.model',
                                'search_read',
                                [],
                                ['name', 'model', 'modules'],
                            ],
                        },
                        id: Math.floor(Math.random() * 100),
                    };
                    const response = (await GenericFunctionsTrigger_1.odooJSONRPCRequest.call(this, body, url));
                    const options = response.result.map((model) => {
                        return {
                            name: model.name,
                            value: model.id,
                            description: `model: ${model.model}<br> modules: ${model.modules}`,
                        };
                    });
                    return options;
                },
            },
            credentialTest: {
                async odooApiTest(credential) {
                    const credentials = credential.data;
                    try {
                        const body = {
                            jsonrpc: '2.0',
                            method: 'call',
                            params: {
                                service: 'common',
                                method: 'login',
                                args: [
                                    (0, GenericFunctionsTrigger_1.odooGetDBName)(credentials === null || credentials === void 0 ? void 0 : credentials.db, credentials === null || credentials === void 0 ? void 0 : credentials.url),
                                    credentials === null || credentials === void 0 ? void 0 : credentials.username,
                                    credentials === null || credentials === void 0 ? void 0 : credentials.password,
                                ],
                            },
                            id: Math.floor(Math.random() * 100),
                        };
                        const options = {
                            headers: {
                                'User-Agent': 'n8n',
                                Connection: 'keep-alive',
                                Accept: '*/*',
                                'Content-Type': 'application/json',
                            },
                            method: 'POST',
                            body,
                            uri: `${(credentials === null || credentials === void 0 ? void 0 : credentials.url).replace(/\/$/, '')}/jsonrpc`,
                            json: true,
                        };
                        const result = await this.helpers.request(options);
                        if (result.error || !result.result) {
                            return {
                                status: 'Error',
                                message: `Credentials are not valid: ${result.error.data.message}`,
                            };
                        }
                    }
                    catch (error) {
                        return {
                            status: 'Error',
                            message: `Settings are not valid: ${error}`,
                        };
                    }
                    return {
                        status: 'OK',
                        message: 'Authentication successful!',
                    };
                },
            },
        };
        this.webhookMethods = {
            default: {
                async checkExists() {
                    var _a;
                    const webhookData = this.getWorkflowStaticData('node');
                    if (webhookData.webhookId === undefined) {
                        return false;
                    }
                    const credentials = await this.getCredentials('odooApi');
                    const url = credentials === null || credentials === void 0 ? void 0 : credentials.url;
                    const username = credentials === null || credentials === void 0 ? void 0 : credentials.username;
                    const password = credentials === null || credentials === void 0 ? void 0 : credentials.password;
                    const db = (0, GenericFunctionsTrigger_1.odooGetDBName)(credentials === null || credentials === void 0 ? void 0 : credentials.db, url);
                    const userID = await GenericFunctionsTrigger_1.odooGetUserID.call(this, db, username, password, url);
                    const response = await GenericFunctionsTrigger_1.odooGet.call(this, db, userID, password, 'base.automation', 'get', url, (_a = webhookData.webhookId) === null || _a === void 0 ? void 0 : _a.toString());
                    const exists = response.result.length > 0;
                    return exists;
                },
                async create() {
                    const webhookUrl = this.getNodeWebhookUrl('default');
                    const webhookData = this.getWorkflowStaticData('node');
                    if (webhookUrl.includes('%20')) {
                        throw new n8n_workflow_1.NodeOperationError(this.getNode(), 'The name of the Odoo Trigger Node is not allowed to contain any spaces!');
                    }
                    const resource = this.getNodeParameter('odooResource');
                    const trigger = this.getNodeParameter('trigger');
                    const credentials = await this.getCredentials('odooApi');
                    const url = credentials === null || credentials === void 0 ? void 0 : credentials.url;
                    const username = credentials === null || credentials === void 0 ? void 0 : credentials.username;
                    const password = credentials === null || credentials === void 0 ? void 0 : credentials.password;
                    const db = (0, GenericFunctionsTrigger_1.odooGetDBName)(credentials === null || credentials === void 0 ? void 0 : credentials.db, url);
                    const userID = await GenericFunctionsTrigger_1.odooGetUserID.call(this, db, username, password, url);
                    const response = await GenericFunctionsTrigger_1.odooCreate.call(this, db, userID, password, 'base.automation', 'create', url, {
                        name: 'Webhook generated by n8n',
                        model_id: resource,
                        state: 'code',
                        trigger: trigger,
                        code: `make_request("POST", "${webhookUrl}", data=(json.dumps(record.read()[0], indent=4, sort_keys=True, default=str)), headers={"Content-Type": "application/json"})`,
                    });
                    webhookData.webhookId = response.result;
                    return true;
                },
                async delete() {
                    var _a;
                    const webhookData = this.getWorkflowStaticData('node');
                    if (webhookData.webhookId === undefined) {
                        return false;
                    }
                    const credentials = await this.getCredentials('odooApi');
                    const url = credentials === null || credentials === void 0 ? void 0 : credentials.url;
                    const username = credentials === null || credentials === void 0 ? void 0 : credentials.username;
                    const password = credentials === null || credentials === void 0 ? void 0 : credentials.password;
                    const db = (0, GenericFunctionsTrigger_1.odooGetDBName)(credentials === null || credentials === void 0 ? void 0 : credentials.db, url);
                    const userID = await GenericFunctionsTrigger_1.odooGetUserID.call(this, db, username, password, url);
                    const response = await GenericFunctionsTrigger_1.odooDelete.call(this, db, userID, password, 'base.automation', 'delete', url, (_a = webhookData.webhookId) === null || _a === void 0 ? void 0 : _a.toString());
                    return response.success;
                },
            },
        };
    }
    async webhook() {
        const returnData = [];
        const bodyData = this.getBodyData();
        if (Array.isArray(bodyData)) {
            returnData.push.apply(returnData, bodyData);
        }
        else {
            returnData.push(bodyData);
        }
        return {
            workflowData: [this.helpers.returnJsonArray(returnData)],
        };
    }
}
exports.OdooTrigger = OdooTrigger;
//# sourceMappingURL=OdooTrigger.node.js.map